package net.jaekl.cfb;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.Locale;

import net.jaekl.cfb.db.CfbSchema;
import net.jaekl.cfb.db.driver.DbDriver;
import net.jaekl.cfb.db.driver.PostgresqlDriver;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class CFB {
	DbDriver m_driver;
	CfbSchema m_schema;
	CfbBundle m_bundle;	
	Locale m_locale;
	
	// Command-line parameters
	String m_dbName; // db name
	String m_host;	// db host
	int m_port;		// db port
	String m_user;	// db user
	String m_pass;	// db password
	
	CFB(Locale locale) {
		m_driver = new PostgresqlDriver();
		m_schema = new CfbSchema(m_driver);
		m_locale = locale;
		m_bundle = CfbBundle.getInst(m_locale);
		
		m_dbName = "CFB";
		m_host = "localhost";
		m_port = 5432;
		m_pass = "";
		m_user = "user";
	}
	
	Options createOptions() {
		Options opt = new Options();
		
		opt.addOption("d", "dbname", true, "DB name");
		opt.addOption("h", "host", true, "DB hostname");
		opt.addOption("p", "pass", true, "DB password");
		opt.addOption("t", "port", true, "DB port");
		opt.addOption("u", "user", true, "DB username");
		
		return opt;
	}
	
	boolean parseArgs(PrintWriter pw, String[] args) {
		Options opt = createOptions();
		
		try {
			CommandLine line = new GnuParser().parse(opt, args);
			if (line.hasOption("d")) {
				m_dbName = line.getOptionValue("d");
			}
			if (line.hasOption("h")) {
				m_host = line.getOptionValue("h");
			}
			if (line.hasOption("p")) {
				m_pass = line.getOptionValue("p");
			}
			if (line.hasOption("t")) {
				m_port = Integer.parseInt(line.getOptionValue("t"));
			}
			if (line.hasOption("u")) {
				m_user = line.getOptionValue("u");
			}
		} 
		catch (ParseException exc) {
			usage(pw, opt);
			return false;
		}
		
		return true;
	}
	
	void usage(PrintWriter pw, Options opt) {
		HelpFormatter help = new HelpFormatter();
		help.printHelp(pw, 80, getClass().getName(), "", opt, 0, 0, "", true);
	}
	
	String trans(String key) {
		return m_bundle.get(key);
	}
	
	void doMain(PrintWriter pw, String[] args) throws SQLException {
		if ( ! parseArgs(pw, args) ) {
			return;
		}
		
		try (Connection con = m_driver.connect(m_host, m_port, m_dbName, m_user, m_pass)) {
			if (null == con) {
				String cannotConnectFormat = trans(CfbBundle.CANNOT_CONNECT);
				String cannotConnect = MessageFormat.format(cannotConnectFormat, m_host, m_port, m_dbName, m_user);
				pw.println(cannotConnect);
				return;
			}
			m_schema.ensureDbInitialized(con);			
		}
	}
	
	public static void main(String[] args) {
		CFB cfb = new CFB(Locale.getDefault());
		
		try (PrintWriter pw = new PrintWriter(System.out)){
			cfb.doMain(pw, args);
		} catch (SQLException exc) {
			exc.printStackTrace();
		}
	}

}
